
#include "ADC_3V3_Display.h"

/*
   (c)2014-2017 Forward Computing and Control Pty. Ltd.
   NSW Australia, www.forward.com.au
   This code is not warranted to be fit for any purpose. You may only use it at your own risk.
   This generated code may be freely used for both private and commercial use
   provided this copyright is maintained.
*/

ADC_3V3_Display::ADC_3V3_Display(pfodDwgs *_dwgsPtr) : pfodControl(_dwgsPtr) {
  z_idx = pfodDwgs::reserveIdx(3);
  baseCol = 0; baseRow = 0; baseScaling = 1.0f;
  value = 0; // initially input
  setBaseScaling(0.3);
  value = 0; // initially Input
  saveCurrentState();
}

void ADC_3V3_Display::setBase(int c, int r) {
  baseCol = c;
  baseRow = r;
}

void ADC_3V3_Display::setBaseScaling(double _scaling) {
  baseScaling = (float)_scaling;
}

void ADC_3V3_Display::skipUpdate() {
  skip_Update = true;
}


void ADC_3V3_Display::setValue(int newValue) {
  int limit = 1023;
  if (newValue < 0) {
    newValue = 0;
  } else if (newValue > limit) {
    newValue = limit;
  }
  pfodControl::setValue(newValue);
}


void ADC_3V3_Display::draw() {
  dwgsPtr->pushZero(baseCol, baseRow, baseScaling);
  if (label != NULL) { // only send label if one set
    dwgsPtr->label().bold().fontSize(0).text(label).color(dwgsPtr->WHITE).offset(4.5, 0).send();
  }
  dwgsPtr->pushZero(8, -1.25, 0.5);
  // scale down 255 to 32 cols
  dwgsPtr->rectangle().filled().color(dwgsPtr->WHITE).size(31, 4).offset(0, -0.75).send();
  dwgsPtr->rectangle().idx(z_idx).filled().color(dwgsPtr->BLACK).size(0, 2.5).send();
  dwgsPtr->popZero();

  dwgsPtr->pushZero(-9, 0);
  dwgsPtr->index().idx(z_idx + 1).send();
  dwgsPtr->popZero();

  dwgsPtr->popZero(); // pop Base push and scale
  fullUpdate(); // update with current state
}

void ADC_3V3_Display::fullUpdate() {
  dwgsPtr->rectangle().idx(z_idx).filled().color(dwgsPtr->BLACK).size(value / 32, 2.5).send();
  dwgsPtr->label().decimals(2).bold().idx(z_idx + 1).intValue(value).units(F("V")).maxValue(1023)
  .displayMax(3.3).color(dwgsPtr->BLACK).send();
}

boolean ADC_3V3_Display::haveStateChange() {
  return ((value != previousValue) && (!skip_Update));
}

void ADC_3V3_Display::saveCurrentState() {
  previousValue = value;
  skip_Update = false;
}

void ADC_3V3_Display::update() {
  if (haveStateChange()) {
    fullUpdate();
  }
  saveCurrentState();
}


