
#include "IO_Pin.h"
/*
   (c)2014-2017 Forward Computing and Control Pty. Ltd.
   NSW Australia, www.forward.com.au
   This code is not warranted to be fit for any purpose. You may only use it at your own risk.
   This generated code may be freely used for both private and commercial use
   provided this copyright is maintained.
*/


IO_Pin::IO_Pin(char _cmd, pfodDwgs *_dwgsPtr) : pfodControl(_dwgsPtr) {
  buttonCmd = _cmd;
  z_idx = pfodDwgs::reserveIdx(5);
  baseCol = 0; baseRow = 0; //baseScaling = 1.0f;
  value = 0; // initially high
  mode = DIGITAL_INPUT;
  showSelected = false;
  forceUpdate = false;
  pwmValue = 0;
  saveCurrentState();
}

void IO_Pin::setBase(int c, int r) {
  baseCol = c;
  baseRow = r;
}


void IO_Pin::forceFullUpdate() {
  forceUpdate = true;
}

int IO_Pin::getMode() {
  return mode;
}

void IO_Pin::setMode(byte m) {
  mode = m;
}

int IO_Pin::getPwmValue() {
  return pwmValue;
}

void IO_Pin::setSelected() {
  showSelected = true;
}

void IO_Pin::clearSelected() {
  showSelected = false;
}

void IO_Pin::setValue(int newValue) {
  int limit = 1;
  if (newValue < 0) {
    newValue = 0;
  } else if (newValue > limit) {
    newValue = limit;
  }
  pfodControl::setValue(newValue);
}

void IO_Pin::setPwmValue(int _pwmValue) {
  pwmValue = _pwmValue;
}

char IO_Pin::getCmd() {
  return buttonCmd;
}

// z_idx == PIN
// z_idx+1 == H/L
// z_idx+2 == text
// z_idx+3 == PWM value text
// z_idx+4 == selected box

void IO_Pin::draw() {
  dwgsPtr->pushZero(baseCol, baseRow, 0.3f);

  dwgsPtr->out->print(F("|t~15~<b>"));
  byte i = (byte)(buttonCmd - 'a');
  if (digitalPinHasPWM(i)) {
    dwgsPtr->out->print(F("&#126;"));
  }
  dwgsPtr->out->print(i);//(int)(buttonCmd - 'a'));
  dwgsPtr->out->print(F("~-4.5~0"));
  dwgsPtr->rectangle().idx(z_idx).color(dwgsPtr->WHITE).centered().rounded().filled().size(2, 2.2).send();
  // make the cols bigger for easy selection
  dwgsPtr->touchZone().cmd(buttonCmd).size(4.5 + 6 + 10, 2.2).offset(-5.5, -1.1).filter(dwgsPtr->TOUCH).send();
  dwgsPtr->touchAction().cmd(buttonCmd).action( // create the action
    dwgsPtr->rectangle().filled().idx(z_idx).rounded().centered().size(2, 2.2).color(dwgsPtr->BLACK) //
  ).send(); // send the touchAction

  dwgsPtr->index().idx(z_idx + 1).send(); // place holder for label
  dwgsPtr->rectangle().idx(z_idx + 4).size(10, 3.4).offset(-7.5, -1.7).color(dwgsPtr->YELLOW).send(); // selection rectangle

  dwgsPtr->pushZero(5.5, 0);
  dwgsPtr->index().idx(z_idx + 3).send(); // place holder
  dwgsPtr->index().idx(z_idx + 2).send(); // place holder
  dwgsPtr->popZero(); // pop Base push and scale

  dwgsPtr->popZero();
  fullUpdate(); // update with current state put update here to check all zeros and offsets set correctly above
}

// every thing here has z_idx
void IO_Pin::fullUpdate() {
  if (showSelected) {
    dwgsPtr->unhide().idx(z_idx + 4).send();
  } else {
    dwgsPtr->hide().idx(z_idx + 4).send();
  }
  if (mode != DIGITAL_PWM) {
    dwgsPtr->label().fontSize(-1).bold().text(value != 0 ? (F("H")) : F("L")).idx(z_idx + 1)
    .color(dwgsPtr->BLACK).send();
  } else {
    dwgsPtr->label().fontSize(-1).bold().text(F("")).idx(z_idx + 1).color(dwgsPtr->BLACK).send();
  }
  if (mode < 0) {
    dwgsPtr->label().idx(z_idx + 2).fontSize(+3).text(F(" ")).offset(0.30, -0.2).color(dwgsPtr->BLACK)
    .send();
  } else if (mode == DIGITAL_INPUT) {
    dwgsPtr->label().idx(z_idx + 2).fontSize(+3).text(F("<")).offset(0.30, -0.2).color(dwgsPtr->BLACK)
    .send();
  } else if (mode == DIGITAL_INPUT_PULLUP) {
    dwgsPtr->label().idx(z_idx + 2).fontSize(+3).text(F("<<-3>\u2191")).offset(0.30, -0.2)
    .color(dwgsPtr->BLACK).send();
  } else if (mode == DIGITAL_OUTPUT) {
    dwgsPtr->label().idx(z_idx + 2).fontSize(+3).text(F(">")).offset(0.30, -0.2).color(dwgsPtr->BLACK)
    .send();

  } else { // DIGITAL_PWM
    dwgsPtr->label().idx(z_idx + 2).fontSize(+3).text(F("&#126;")).offset(0.30, -0.2).color(dwgsPtr->BLACK)
    .send();
  }
  // pwm value
  if (mode < DIGITAL_PWM) {
    // not pwm
    dwgsPtr->hide().idx(z_idx + 3).send();
  } else {
    dwgsPtr->label().fontSize(+1).intValue(getPwmValue()).units(F("%")).decimals(1).maxValue(255)
    .displayMax(100).idx(z_idx + 3).offset(6, -0.2).color(dwgsPtr->BLACK).send();
    dwgsPtr->unhide().idx(z_idx + 3).send();
  }

}

boolean IO_Pin::haveStateChange() {
  return ((mode != previousMode) || (pwmValue != previousPwmValue) || (value != previousValue)
          || (previousShowSelected != showSelected) || forceUpdate);
}

void IO_Pin::saveCurrentState() {
  previousMode = mode;
  previousPwmValue = pwmValue;
  previousValue = value; // h/l
  previousShowSelected = showSelected;
  forceUpdate = false;
}

void IO_Pin::update() {
  if (haveStateChange()) {
    fullUpdate();
  }
  saveCurrentState();
}

